/***********************/
/* interlace.c 1.00    */
/* for TestLace        */
/* by Adam M. Costello */
/* (amc@cs.wustl.edu)  */
/***********************/

/* This is ANSI C plus X11R5 code. */


#include "fail.h"
#include "lacescheme.h"
#include "visit.h"

#include <stdio.h>
#include <stdlib.h>


void usage(void)
/* Prints a usage message to stderr, then exits unsuccessfully. */
{
  fputs(
    "Options:\n"
    , stderr);

  laceopts(stderr);

  fail(
    "           <ppmfilename>   Read image from a file (default is stdin)\n"
    "\n"
    "Note:  Input must be a rawbit PPM file (P6).\n"
    "\n"
    );
}


void parseargs(int argc, const char **argv,
               schemeimpl *dolace, const char **ppmfilename)
/* Parses the command line arguments and sets *dolace and  */
/* *ppmfilename if they are specified on the command line. */
/* Does not modify variables whose options are not given.  */
/* Calls fail() if the arguments are improper.             */
{
  schemeimpl tmp;

  while (*++argv) {
    tmp = lacescheme(*argv);
    if (tmp) {
      *dolace = tmp;
    } else if (**argv != '-') {
      *ppmfilename = *argv;
      if (argv[1]) usage();
    } else {
      usage();
    }
  }
}


int main(int argc, const char **argv)
{
  int width, height, maxval;
  const char *ppmfilename = NULL;
  schemeimpl dolace = defaultscheme;
  FILE *ppmfile = NULL;
  IMAGE *image = NULL;

  parseargs(argc, argv, &dolace, &ppmfilename);

  if (!ppmfilename) ppmfile = stdin;
  else {
    ppmfile = fopen(ppmfilename, "rb");
    if (!ppmfile) fail("Cannot open input file.\n");
  }

  if (fgetc(ppmfile) != 'P' || fgetc(ppmfile) != '6')
    fail("Input is not a P6 file.\n");

  fscanf(ppmfile, "%d %d %d", &width, &height, &maxval);
  fgetc(ppmfile);

  image = loadimage(ppmfile,width,height);
  close(ppmfile);
  ppmfile = NULL;

  printf("P6\n%d %d\n%d\n", width, height, maxval);

  useimage(image);
  dolace(width,height);

  return 0;
}
