/* adam7cross [ <ppmfile> ] */
/* Outputs the result of reordering the pixels in <ppmfile> (or    */
/* stdin) according to Adam's seven-pass scheme and performing the */
/* cross filter on each subimage separately.                       */

#include <stdio.h>
#include <stdlib.h>

static unsigned int width, height, mod;
static unsigned char (*pixels)[3];
#define pixelbyte(row,col,byte) pixels[(row) * width + (col)][byte]

void usage(char **argv)
{
  fprintf(stderr, "usage:\n%s <ppmfile>\n", argv[0]);
  exit(1);
}

void visit(unsigned int row, unsigned int col,
           unsigned int rowstep, unsigned int colstep)
{
  unsigned int byte, lowright, upleft, lowleft, upright;

  for (byte = 0;  byte < 3;  ++byte) {
    lowright = pixelbyte(row,col,byte);
    upleft = row >= rowstep && col >= colstep  ?
               pixelbyte(row - rowstep, col - colstep, byte) : 0;
    lowleft = col >= colstep  ?  pixelbyte(row, col - colstep, byte) : 0;
    upright = row >= rowstep  ?  pixelbyte(row - rowstep, col, byte) : 0;
    putchar((mod + mod + lowright + upleft - lowleft - upright) % mod);
  }
}

main(int argc, char **argv)
{
  unsigned int max, step, row, col, byte;
  char *ppmfilename = NULL;
  FILE *ppmfile;

  if (argc == 1) ppmfile = stdin;
  else if (argc == 2) ppmfilename = argv[1];
  else usage(argv);

  if (ppmfilename) {
    ppmfile = fopen(ppmfilename, "rb");
    if (!ppmfile) {
      fprintf(stderr, "Cannot open %s\n", ppmfilename);
      exit(1);
    }
  }

  if (fgetc(ppmfile) != 'P' || fgetc(ppmfile) != '6') {
    fprintf(stderr, "%s is not a P6 file.\n", ppmfilename);
    exit(1);
  }

  fscanf(ppmfile, "%d %d %d", &width, &height, &max);
  mod = max + 1;
  fgetc(ppmfile);

  pixels = malloc(width * height * 3);
  if (!pixels) {
    fputs("Out of memory\n", stderr);
    exit(1);
  }

  for (row = 0;  row < height;  ++row)
    for (col = 0;  col < width;  ++col)
      for (byte = 0;  byte < 3;  ++byte)
        pixelbyte(row,col,byte) = getc(ppmfile);

  printf("P6\n%d %d\n%d\n", width, height, max);

  for (row = 0;  row < height;  row += 8)
    for (col = 0;  col < width;  col += 8)
      visit(row,col,8,8);
  for (row = 0;  row < height;  row += 8)
    for (col = 4;  col < width;  col += 8)
      visit(row,col,8,8);
  for (row = 4;  row < height;  row += 8)
    for (col = 0;  col < width;  col += 4)
      visit(row,col,8,4);
  for (row = 0;  row < height;  row += 4)
    for (col = 2;  col < width;  col += 4)
      visit(row,col,4,4);
  for (row = 2;  row < height;  row += 4)
    for (col = 0;  col < width;  col += 2)
      visit(row,col,4,2);
  for (row = 0;  row < height;  row += 2)
    for (col = 1;  col < width;  col += 2)
      visit(row,col,2,2);
  for (row = 1;  row < height;  row += 2)
    for (col = 0;  col < width;  col += 1)
      visit(row,col,2,1);
}
