/* adam7demo <ppmfile> */
/* Creates 7 files, <ppmfile>.1 through <ppmfile>.7, which show the   */
/* partial image after each pass of the Adam7 interlace scheme.  Note */
/* that <ppmfile>.7 should be identical to <ppmfile>, except possibly */
/* for the choice of white characters in the header.                  */
/*                   */
/*  1 6 4 6 2 6 4 6  */
/*  7 7 7 7 7 7 7 7  */
/*  5 6 5 6 5 6 5 6  */
/*  7 7 7 7 7 7 7 7  */
/*  3 6 4 6 3 6 4 6  */
/*  7 7 7 7 7 7 7 7  */
/*  5 6 5 6 5 6 5 6  */
/*  7 7 7 7 7 7 7 7  */

#include <stdio.h>
#include <stdlib.h>

static unsigned int width, height, max;
static unsigned char (*pixels)[3];
static unsigned char (*newpixels)[3];
#define pixelbyte(row,col,byte) pixels[(row) * width + (col)][byte]
#define newpixelbyte(row,col,byte) newpixels[(row) * width + (col)][byte]

void usage(char **argv)
{
  fprintf(stderr, "usage:\n%s <ppmfile>\n", argv[0]);
  exit(1);
}

void visit(unsigned int row, unsigned int col, unsigned int w, unsigned int h)
{
  unsigned int r, endr, c, endc, byte;

  endr = row + h;
  if (endr > height) endr = height;

  endc = col + w;
  if (endc > width) endc = width;

  for (r = row;  r < endr;  ++r)
    for (c = col;  c < endc;  ++c)
      for (byte = 0;  byte < 3;  ++byte)
        newpixelbyte(r,c,byte) = pixelbyte(row,col,byte); 
}

void putimage(char *infilename, int pass)
{
  char outfilename[256];
  FILE *outfile;
  unsigned int row, col, byte;

  sprintf(outfilename, "%.253s.%d", infilename, pass % 10);
  outfile = fopen(outfilename, "wb");
  if (!outfile) {
    fprintf(stderr, "Unable to write to %s\n", outfilename);
    return;
  }

  fprintf(outfile, "P6\n%d %d\n%d\n", width, height, max);

  for (row = 0;  row < height;  ++row)
    for (col = 0;  col < width;  ++col)
      for (byte = 0;  byte < 3;  ++byte)
        fputc(newpixelbyte(row,col,byte), outfile);

  fclose(outfile);
}

main(int argc, char **argv)
{
  unsigned int step, row, col, byte;
  char *ppmfilename = NULL;
  FILE *ppmfile;

  if (argc == 2) ppmfilename = argv[1];
  else usage(argv);

  if (ppmfilename) {
    ppmfile = fopen(ppmfilename, "rb");
    if (!ppmfile) {
      fprintf(stderr, "Cannot open %s\n", ppmfilename);
      exit(1);
    }
  }

  if (fgetc(ppmfile) != 'P' || fgetc(ppmfile) != '6') {
    fprintf(stderr, "%s is not a P6 file.\n", ppmfilename);
    exit(1);
  }

  fscanf(ppmfile, "%d %d %d", &width, &height, &max);
  fgetc(ppmfile);

  pixels = malloc(width * height * 3);
  newpixels = malloc(width * height * 3);
  if (!pixels || !newpixels) {
    fputs("Out of memory\n", stderr);
    exit(1);
  }

  for (row = 0;  row < height;  ++row)
    for (col = 0;  col < width;  ++col)
      for (byte = 0;  byte < 3;  ++byte)
        pixelbyte(row,col,byte) = getc(ppmfile);

  fclose(ppmfile);

  for (row = 0;  row < height;  row += 8)
    for (col = 0;  col < width;  col += 8)
      visit(row,col,8,8);

  putimage(ppmfilename,1);

  for (row = 0;  row < height;  row += 8)
    for (col = 4;  col < width;  col += 8)
      visit(row,col,4,8);

  putimage(ppmfilename,2);

  for (row = 4;  row < height;  row += 8)
    for (col = 0;  col < width;  col += 4)
      visit(row,col,4,4);

  putimage(ppmfilename,3);

  for (row = 0;  row < height;  row += 4)
    for (col = 2;  col < width;  col += 4)
      visit(row,col,2,4);

  putimage(ppmfilename,4);

  for (row = 2;  row < height;  row += 4)
    for (col = 0;  col < width;  col += 2)
      visit(row,col,2,2);

  putimage(ppmfilename,5);

  for (row = 0;  row < height;  row += 2)
    for (col = 1;  col < width;  col += 2)
      visit(row,col,1,2);

  putimage(ppmfilename,6);

  for (row = 1;  row < height;  row += 2)
    for (col = 0;  col < width;  col += 1)
      visit(row,col,1,1);

  putimage(ppmfilename,7);
}
