/* ppmtoppim <ppmfile> <mapfile> */
/* Outputs a PPIM (Proprietary Pixel Index Map) file, which is just a */
/* PGM file, but the values are not grey values, but indices into the */
/* color map in <mapfile> (which should have been created by ppmtomap */
/* <ppmfile>).                                                        */

#include <stdio.h>
#include <stdlib.h>

main(int argc, char **argv)
{
  unsigned int width, height, max, w, h, m, numcolors, index, row, col,
               red, green, blue;
  FILE *ppmfile, *mapfile;
  unsigned char map[256][3];

  if (argc != 3) {
    fprintf(stderr, "usage:\n%s <ppmfile> <mapfile>\n", argv[0]);
    exit(1);
  }

  ppmfile = fopen(argv[1], "rb");
  mapfile = fopen(argv[2], "rb");
  if (!ppmfile || !mapfile) {
    fputs("Cannot open files.\n", stderr);
    exit(1);
  }

  if (fgetc(ppmfile) != 'P' || fgetc(ppmfile) != '6') {
    fprintf(stderr, "%s is not a P6 file.\n", argv[1]);
    exit(1);
  }

  fscanf(ppmfile, "%d %d %d", &width, &height, &max);
  fgetc(ppmfile);

  if (fgetc(mapfile) != 'P' || fgetc(mapfile) != '6') {
    fprintf(stderr, "%s is not a P6 file.\n", argv[2]);
    exit(1);
  }

  fscanf(mapfile, "%d %d %d", &w, &h, &m);
  fgetc(mapfile);

  numcolors = w * h;

  if (numcolors > 256) {
    fputs("Sorry, cannot handle more than 256 colors.\n", stderr);
    exit(1);
  }

  for (index = 0;  index < numcolors;  ++index) {
    map[index][0] = getc(mapfile);
    map[index][1] = getc(mapfile);
    map[index][2] = getc(mapfile);
  }

  printf("P5\n%d %d\n%d\n", width, height, numcolors - 1);

  for (row = 0;  row < height;  ++row)
    for (col = 0;  col < width;  ++col) {
      red = getc(ppmfile);
      green = getc(ppmfile);
      blue = getc(ppmfile);
      for (index = 0;  index < numcolors;  ++index)
        if (   map[index][0] == red
            && map[index][1] == green
            && map[index][2] == blue ) break;
      if (index >= numcolors) {
        fprintf(stderr, "%s uses a color not in %s\n", argv[1], argv[2]);
        exit(1);
      }
      putchar(index);
    }
}
