/* spersecross [ -i <iterations> ] <ppmfile> */
/* Outputs the result of reordering the pixels in  */
/* <ppmfile> according to the intersperse/cross    */
/* scheme, which is currently defined by this very */
/* inefficient but readable code.                  */

#include <stdio.h>
#include <stdlib.h>

static unsigned int width, height, mod;
static unsigned char (*pixels)[3];
#define pixelbyte(row,col,byte) pixels[(row) * width + (col)][byte]

void usage(char **argv)
{
  fprintf(stderr, "usage:\n%s [ -i <iterations> ] <ppmfile>\n", argv[0]);
  exit(1);
}

void visit(unsigned int row, unsigned int col, unsigned int step)
{
  unsigned int byte, lowright, upleft, lowleft, upright;

  for (byte = 0;  byte < 3;  ++byte) {
    lowright = pixelbyte(row,col,byte);
    upleft = row >= step && col >= step  ?
               pixelbyte(row - step, col - step, byte) : 0;
    lowleft = col >= step  ?  pixelbyte(row, col - step, byte) : 0;
    upright = row >= step  ?  pixelbyte(row - step, col, byte) : 0;
    putchar((mod + mod + lowright + upleft - lowleft - upright) % mod);
  }
}

main(int argc, char **argv)
{
  unsigned int iterations, firststep, max, step, row, col, byte;
  char **arg, *ppmfilename;
  FILE *ppmfile;

  iterations = 15;  /* This needs to be large enough that 2^iterations */
                    /* is larger than width and height.                */

  if (argc < 2) usage(argv);
  for (arg = argv + 1;  *arg;  ++arg) {
    if (strcmp(*arg, "-i")) break;
    if (!*++arg) usage(argv);
    iterations = atoi(*arg);
  }
  if (!*arg) usage(argv);
  ppmfilename = *arg;

  ppmfile = fopen(ppmfilename, "rb");
  if (!ppmfile) {
    fprintf(stderr, "Cannot open %s\n", ppmfilename);
    exit(1);
  }

  if (fgetc(ppmfile) != 'P' || fgetc(ppmfile) != '6') {
    fprintf(stderr, "%s is not a P6 file.\n", ppmfilename);
    exit(1);
  }

  fscanf(ppmfile, "%d %d %d", &width, &height, &max);
  mod = max + 1;
  fgetc(ppmfile);

  pixels = malloc(width * height * 3);
  if (!pixels) {
    fputs("Out of memory\n", stderr);
    exit(1);
  }

  for (row = 0;  row < height;  ++row)
    for (col = 0;  col < width;  ++col)
      for (byte = 0;  byte < 3;  ++byte)
        pixelbyte(row,col,byte) = getc(ppmfile);

  printf("P6\n%d %d\n%d\n", width, height, max);

  firststep = 1 << iterations;

  for (row = 0;  row < height;  row += firststep)
    for (col = 0;  col < width;  col += firststep)
      visit(row,col,firststep);
  for (step = firststep;  step > 1;  step /= 2) {
    for (row = step/2;  row < height;  row += step)
      for (col = step/2;  col < width;  col += step)
        visit(row,col,step);
    for (row = 0;  row < height;  row += step)
      for (col = step/2;  col < width;  col += step)
        visit(row,col,step);
    for (row = step/2;  row < height;  row += step)
      for (col = 0;  col < width;  col += step)
        visit(row,col,step);
  }
}
