/* sub <ppmfile> */
/* Outputs the result of passing <ppmfile> through the SUB filter as  */
/* defined in the 4th draft of the PBF spec. <ppmfile> may also be a  */
/* ppma file (type "p7").                                             */

#include <stdio.h>
#include <stdlib.h>

main(int argc, char **argv)
{
  int type1, type2, width, height, max, havealpha, col, row, byte;
  unsigned int mod;
  FILE *ppmfile;
  unsigned char pixelspace[4], prevpixelspace[4], *pixel = pixelspace,
                *prevpixel = prevpixelspace, *tmp;

  if (argc != 2) {
    fprintf(stderr,
            "usage:\n%s <ppmfile>\n",
            argv[0]);
    exit(1);
  }

  ppmfile = fopen(argv[1], "rb");
  if (!ppmfile) {
    fprintf(stderr, "Cannot open file %s\n", argv[1]);
    exit(1);
  }

  type1 = fgetc(ppmfile);
  type2 = fgetc(ppmfile);
  if (type1 == 'P' && type2 == '6') havealpha = 0;
  else if (type1 == 'p' && type2 == '7') havealpha = 1;
  else {
    fprintf(stderr, "%s is not a P6 or p7 file.\n", argv[1]);
    exit(1);
  }

  fscanf(ppmfile, "%d %d %d", &width, &height, &max);
  mod = max + 1;
  fgetc(ppmfile);

  printf("%c%c\n%d %d\n%d\n", type1, type2, width, height, max);

  for (row = 0;  row < height;  ++row) {
    prevpixel[0] = prevpixel[1] = prevpixel[2] = prevpixel[3] = 0;
    for (col = 0;  col < width;  ++col) {
      for (byte = 0;  byte < 3 + havealpha;  ++byte) {
        pixel[byte] = getc(ppmfile);
        putchar((mod + pixel[byte] - prevpixel[byte]) % mod);
      }
      tmp = prevpixel;
      prevpixel = pixel;
      pixel = tmp;
    }
  }
}
